InitSprite()
InitKeyboard()

XIncludeFile "GL.pbi"
XIncludeFile "GLEXT.pbi"

; Imports
Import "OpenGL32.lib"
  glClearDepth    (depth.d)                                             As "_glClearDepth@8"
  glClipPlane     (plane.l, equation.d)                                 As "_glClipPlane@12"
  glDepthRange    (near.d, far.d)                                       As "_glDepthRange@16"
  glFrustum       (left.d, right.d, bottom.d, top.d, znear.d, zfar.d)   As "_glFrustum@48"
  glGetClipPlane  (plane.l, equation.d)                                 As "_glGetClipPlane@8"
  glOrtho         (left.d, right.d, bottom.d, top.d, znear.d, zfar.d)   As "_glOrtho@48"
EndImport

Import "Glu32.lib"
  gluCylinder     (*qobj, baseRadius.d, topRadius.d, height.d, slices.l, stacks.l)                      As "_gluCylinder@36"
  gluDisk         (*qobj, innerRadius.d, outerRadius.d, slices.l, loops.l)                              As "_gluDisk@28"
  gluLookAt       (eyex.d, eyey.d, eyez.d, centerx.d, centery.d, centerz.d, upx.d, upy.d, upz.d)        As "_gluLookAt@72"
  gluOrtho2D      (left.d, right.d, bottom.d, top.d)                                                    As "_gluOrtho2D@32"
  gluPartialDisk  (*qobj, innerRadius.d, outerRadius.d, slices.l, loops.l, startAngle.d, sweepAngle.d)  As "_gluPartialDisk@44"
  gluPerspective  (fovy.d, aspect.d, zNear.d, zFar.d)                                                   As "_gluPerspective@32"
  gluPickMatrix   (x.d, y.d, width.d, height.d, *viewport)                                              As "_gluPickMatrix@36"
  gluProject      (objx.d, objy.d, objz.d, *modelMatrix, *projMatrix, *viewport, *winx, *winy, *winz)   As "_gluProject@48"
  gluSphere       (*qobj, radius.d, slices.l, stacks.l)                                                 As "_gluSphere@20"
  gluTessNormal   (*tess, x.d, y.d, z.d)                                                                As "_gluTessNormal@28"
  gluTessProperty (*tess, which.l, value.l)                                                             As "_gluTessProperty@12"
  gluUnProject    (winx.d, winy.d, winz.d, *modelMatrix, *projMatrix, *viewport, *objx, *objy, *objz)   As "_gluUnProject@48"
EndImport

; Error handling
Procedure.s GetLastError()
  Shared lastErrorMessage.s

  ProcedureReturn lastErrorMessage
EndProcedure

Procedure CompileShader(glslContext.l, type.l, *source.STRING, size.l)
  Protected result.l
  Protected status.l
  Protected length.l
  Protected error.s
  Shared lastErrorMessage.s

  result = 0
  status = 0

  If glslContext <> 0
    If *source <> 0 And size > 0
      
      result = glCreateShaderObject_(type)
      If result <> 0
        
        glShaderSource_(result, 1, @*source, @size)
        glCompileShader_(result)
        glGetObjectParameteriv_(result, #GL_OBJECT_COMPILE_STATUS_ARB, @status)
        glAttachObject_(glslContext, result)
        
        If status = 0
          
          glGetObjectParameteriv_(result, #GL_OBJECT_INFO_LOG_LENGTH_ARB, @length)
          error.s = Space(length)
          glGetInfoLog_(result, length, @size, @error)
          lastErrorMessage = "Compiler error:"+Chr(10)+Chr(10)+error
          
          glDeleteObject_(result)
          result = 0
          
        EndIf

      Else
        lastErrorMessage = "Couldn't create shader object."
      EndIf

    Else
      lastErrorMessage = "No source found."
    EndIf
  Else
    lastErrorMessage = "No shadercontext found."
  EndIf

  ProcedureReturn result
EndProcedure

Procedure LinkShader(glslContext.l)
  Protected result.l
  Protected status.l
  Protected length.l
  Protected size.l
  Protected error.s
  Shared lastErrorMessage.s

  result = 0
  status = 0

  If glslContext <> 0

    glLinkProgram_(glslContext)
    glGetObjectParameteriv_(glslContext, #GL_OBJECT_LINK_STATUS_ARB, @status)

    If status = 0

      glGetObjectParameteriv_(glslContext, #GL_OBJECT_INFO_LOG_LENGTH_ARB, @length)
      error.s = Space(length)
      glGetInfoLog_(glslContext, length, @size, @error)
      lastErrorMessage = "Linker error:"+Chr(10)+Chr(10)+error

    Else
      result = 1
    EndIf

  Else
    lastErrorMessage = "No shadercontext found."
  EndIf

  ProcedureReturn result
EndProcedure

OpenWindow(0, 0, 0, 640, 480, "Shader Test")
OpenWindowedScreen(WindowID(0), 0, 0, 640, 480, 1, 0, 0)

LoadSprite(0, #PB_Compiler_Home + "examples/sources/data/geebee2.bmp")

Extensions.s = PeekS(glGetString_(#GL_EXTENSIONS))

If FindString(Extensions, "GL_ARB_shading_language_100", 1) <> 0 And FindString(Extensions, "GL_ARB_shader_objects", 1) <> 0
  InitGLSLShader()
  
  glslContext = glCreateProgramObject_()
  If glslContext
    
    glslVShader = CompileShader(glslContext, #GL_VERTEX_SHADER_ARB, ?VShader, ?VShader_End - ?VShader)
    If glslVShader
      glslFShader = CompileShader(glslContext, #GL_FRAGMENT_SHADER_ARB, ?FShader, ?FShader_End - ?FShader)
      If glslFShader
        
        If LinkShader(glslContext) = 0
          Goto ERROR
        EndIf
        
      Else
        Goto ERROR
      EndIf
    Else
      Goto ERROR
    EndIf
    
    glUseProgramObject_(glslContext)
    
    glUniform1i_(glGetUniformLocation_(glslContext, @"tex"), 0)
    
  Else
    
    lastErrorMessage = "Couldn't create shadercontext."
    Goto ERROR
    
  EndIf
  
Else
  
  lastErrorMessage = "This graphics card can't use high level shaders."
  Goto ERROR
  
EndIf

switch = glslContext

glEnable_(#GL_BLEND)
glEnable_(#GL_ALPHA_TEST)
glBlendFunc_(#GL_SRC_ALPHA, #GL_ONE)
glAlphaFunc_(#GL_GREATER, 0.0)

Repeat
  ClearScreen(RGB(0, 64, 0))
  DisplaySprite(0, 64, 64)
  FlipBuffers()
  
  ExamineKeyboard()
  If KeyboardReleased(#PB_Key_1)
    If switch = 0 : switch = glslContext : Else : switch = 0 : EndIf
    glUseProgramObject_(switch)
  EndIf
  
  Delay(1)
Until WindowEvent() = #PB_Event_CloseWindow Or KeyboardReleased(#PB_Key_Escape)

glDisable_(#GL_ALPHA_TEST)
glDisable_(#GL_BLEND)

If glslVShader
  glDeleteObject_(glslVShader)
EndIf
If glslFShader
  glDeleteObject_(glslFShader)
EndIf
If glslContext
  glDeleteObject_(glslContext)
EndIf
End

DataSection
  VShader:
  Data.s "void main() {" + Chr(10) + "  gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;" + Chr(10) + "  gl_TexCoord[0] = gl_MultiTexCoord0 * gl_TextureMatrix[0];" + Chr(10) +  "}"
  VShader_End:
  
  FShader:
  Data.s "uniform sampler2DRect tex;" + Chr(10) + "void main() {" + Chr(10) + "  gl_FragColor = texture2DRect(tex, vec2(gl_TexCoord[0]));" + Chr(10) + "  gl_FragColor.a = sqrt(pow(gl_FragColor.r - 1.0, 2.0) + pow(gl_FragColor.g - 0.0, 2.0) + pow(gl_FragColor.b - 1.0, 2.0));" + Chr(10) + "}" + Chr(10)
  FShader_End:
EndDataSection

ERROR:
MessageRequester("ERROR", GetLastError())
End
; IDE Options = PureBasic 4.30 (Windows - x86)
; FirstLine = 105
; Folding = -
; SubSystem = opengl
; DisableDebugger